/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *            preferences.c
 *
 *  Wed Jan 22 23:45:46 2003
 *  Copyright  2003  luke@biddell.co.uk
 *  luke@dhcp-45-369
 ****************************************************************************/


#include "preferences.h"
#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/utsname.h>
#include <gconf/gconf-client.h>
#include "gbcommon.h"


/* Application config */
static PreferencesData appPrefs = 
	{"0,0,0", "0,0,0", 4, FALSE, TRUE, TRUE, "/tmp", TRUE, FALSE, FALSE, FALSE, 
		TRUE, FALSE, "default"};
GConfClient* gconf_client = NULL;
	
#define GB_BASE_KEY "/apps/GnomeBaker/"
#define GB_READER 			GB_BASE_KEY "ReadDevice"
#define GB_WRITER 			GB_BASE_KEY "WriteDevice"
#define GB_WRITE_SPEED 		GB_BASE_KEY "WriteSpeed"
#define GB_MULTI_SESSION 	GB_BASE_KEY "MultiSession"
#define GB_DUMMY 			GB_BASE_KEY "Dummy"
#define GB_EJECT 			GB_BASE_KEY "Eject"
#define GB_TEMP_DIR 		GB_BASE_KEY "TempDir"
#define GB_FAST_BLANK 		GB_BASE_KEY "FastBlank"
#define GB_BOOTABLE 		GB_BASE_KEY "DataCDBootable"
#define GB_BURNFREE 		GB_BASE_KEY "BurnFree"
#define GB_SHOWHIDDEN 		GB_BASE_KEY "ShowHiddenFiles"
#define GB_CLEANTEMPDIR 	GB_BASE_KEY "CleanTempDir"
#define GB_CREATEISOONLY 	GB_BASE_KEY "CreateISOOnly"
#define GB_WRITE_MODE 		GB_BASE_KEY "WriteMode"

void preferences_create_dir(const gchar * const filename);



PreferencesData *
preferences_get()
{
	GB_LOG_FUNC
	return &appPrefs;
}


gboolean
preferences_init()
{
	GB_LOG_FUNC
	
	gboolean ok = FALSE;
	
	gconf_client = gconf_client_get_default();
	if(gconf_client == NULL) 
		g_critical("Failed to initialise gconf");
	else
		ok = preferences_load();
	
	return ok;
}


gboolean
preferences_save()
{
	GB_LOG_FUNC

	gboolean ok = TRUE;
	
	gconf_client_set_string(gconf_client, GB_READER, appPrefs.readDevice, NULL);
	gconf_client_set_string(gconf_client, GB_WRITER, appPrefs.writeDevice, NULL);
	gconf_client_set_int(gconf_client, GB_WRITE_SPEED, appPrefs.speed, NULL);
	gconf_client_set_bool(gconf_client, GB_MULTI_SESSION, appPrefs.multisession, NULL);
	gconf_client_set_bool(gconf_client, GB_EJECT, appPrefs.eject, NULL);
	gconf_client_set_bool(gconf_client, GB_DUMMY, appPrefs.dummy, NULL);
	gconf_client_set_string(gconf_client, GB_TEMP_DIR, appPrefs.tempDir, NULL);
	gconf_client_set_bool(gconf_client, GB_FAST_BLANK, appPrefs.fastBlank, NULL);
	gconf_client_set_bool(gconf_client, GB_BOOTABLE, appPrefs.datacdbootable, NULL);
	gconf_client_set_bool(gconf_client, GB_BURNFREE, appPrefs.burnfree, NULL);
	gconf_client_set_bool(gconf_client, GB_SHOWHIDDEN, appPrefs.showhiddenfiles, NULL);
	gconf_client_set_bool(gconf_client, GB_CLEANTEMPDIR, appPrefs.cleantempdir, NULL);
	gconf_client_set_bool(gconf_client, GB_CREATEISOONLY, appPrefs.createisoonly, NULL);
	gconf_client_set_string(gconf_client, GB_WRITE_MODE, appPrefs.writeMode, NULL);
	
	/* also make sure that the tmp dir exists */
	gchar* maketempdir = g_strdup_printf("mkdir %s", appPrefs.tempDir);
	system(maketempdir);
	g_free(maketempdir);

	return ok;
}


gboolean
preferences_load()
{
	GB_LOG_FUNC

	gboolean ok = TRUE;
	
	if(gconf_client_get_string(gconf_client, GB_READER, NULL) != NULL)
	{
		gchar* reader = gconf_client_get_string(gconf_client, GB_READER, NULL);
		if(reader != NULL)
		{
			strncpy(appPrefs.readDevice, reader, 64);		
			g_free(reader);
		}
		
		gchar* writer = gconf_client_get_string(gconf_client, GB_WRITER, NULL);
		if(writer != NULL)
		{
			strncpy(appPrefs.writeDevice, writer, 64);
			g_free(writer);
		}
		
		appPrefs.speed = gconf_client_get_int(gconf_client, GB_WRITE_SPEED, NULL);
		appPrefs.speed = gconf_client_get_int(gconf_client, GB_WRITE_SPEED, NULL);
		appPrefs.multisession = gconf_client_get_bool(gconf_client, GB_MULTI_SESSION, NULL);
		appPrefs.eject = gconf_client_get_bool(gconf_client, GB_EJECT, NULL);
		appPrefs.dummy = gconf_client_get_bool(gconf_client, GB_DUMMY, NULL);
		
		gchar* tmpdir = gconf_client_get_string(gconf_client, GB_TEMP_DIR, NULL);
		if(tmpdir != NULL)
		{
			strncpy(appPrefs.tempDir, tmpdir, 256);
			g_free(tmpdir);
		}
		
		appPrefs.fastBlank = gconf_client_get_bool(gconf_client, GB_FAST_BLANK, NULL);		
		appPrefs.datacdbootable = gconf_client_get_bool(gconf_client, GB_BOOTABLE, NULL);
		appPrefs.burnfree = gconf_client_get_bool(gconf_client, GB_BURNFREE, NULL);
		appPrefs.showhiddenfiles = gconf_client_get_bool(gconf_client, GB_SHOWHIDDEN, NULL);		
		appPrefs.cleantempdir = gconf_client_get_bool(gconf_client, GB_CLEANTEMPDIR, NULL);		
		appPrefs.createisoonly = gconf_client_get_bool(gconf_client, GB_CREATEISOONLY, NULL);		
		
		gchar* mode = gconf_client_get_string(gconf_client, GB_WRITE_MODE, NULL);
		if(mode != NULL)
		{
			strncpy(appPrefs.writeMode, mode, 64);
			g_free(mode);
		}
	}
	else
	{	
		gchar* str = g_build_filename("/tmp/GnomeBaker", g_get_user_name(), NULL);
		strncpy(appPrefs.tempDir, str, 255);
		g_free(str);
		
		ok = preferences_save();
	}
	
	return ok;
}


gchar* 
preferences_get_copy_data_cd_image()
{
	GB_LOG_FUNC
	return g_build_filename(appPrefs.tempDir, "gnomebaker_copy_data_cd.iso", NULL);
}


gchar* 
preferences_get_create_data_cd_image()
{
	GB_LOG_FUNC
	return g_build_filename(appPrefs.tempDir, "gnomebaker_create_data_cd.iso", NULL);
}


gchar* 
preferences_get_convert_audio_track_dir()
{
	GB_LOG_FUNC
	
	gchar* path = g_build_filename(appPrefs.tempDir, "create_audiocd", NULL);
	gchar* maketempdir = g_strdup_printf("mkdir %s", path);
	system(maketempdir);
	g_free(maketempdir);
	return path;
}



GtkToolbarStyle  
preferences_get_toolbar_style()
{
	GB_LOG_FUNC
	GtkToolbarStyle ret = GTK_TOOLBAR_BOTH;
	
	gchar* style = gconf_client_get_string(gconf_client, 
		"/desktop/gnome/interface/toolbar_style", NULL);
	g_return_val_if_fail(style != NULL, ret);		
			
	if(g_ascii_strcasecmp(style, "both_horiz") == 0)
		ret = GTK_TOOLBAR_BOTH_HORIZ;
	else if(g_ascii_strcasecmp(style, "icons") == 0)
		ret = GTK_TOOLBAR_ICONS;
	else if(g_ascii_strcasecmp(style, "text") == 0)
		ret = GTK_TOOLBAR_TEXT;
	
	g_message( "toolbar style [%s] [%d]", style, ret);
	g_free(style);
	
	return ret;
}
