/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 /*
 * File: prefsdlg.h
 * Created by: luke@biddell.co.uk
 * Created on: Fri Apr  9 12:04:30 2004
 */

#include "prefsdlg.h"
#include "preferences.h"
#include "gbcommon.h"
#include "filebrowser.h"

void prefsdlg_on_ok(GtkButton* button, gpointer user_data);


GladeXML* prefsdlg_xml = NULL;


GtkWidget* 
prefsdlg_new(void)
{	
	GB_LOG_FUNC	
	prefsdlg_xml = glade_xml_new(glade_file, widget_prefsdlg, NULL);
	glade_xml_signal_autoconnect(prefsdlg_xml);		
	
	PreferencesData *appPrefs =(PreferencesData*)preferences_get();	
	GtkWidget* entry = glade_xml_get_widget(prefsdlg_xml, widget_prefsdlg_tempdir);
	gtk_entry_set_text(GTK_ENTRY(entry), appPrefs->tempDir);
	
	GtkWidget* checkCleanTmp = 	glade_xml_get_widget(prefsdlg_xml, widget_prefsdlg_cleantempdir);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkCleanTmp), appPrefs->cleantempdir);
	
	GtkWidget* checkShowHidden = glade_xml_get_widget(prefsdlg_xml, widget_prefsdlg_showhidden);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkShowHidden), appPrefs->showhiddenfiles);
	
	return glade_xml_get_widget(prefsdlg_xml, widget_prefsdlg);
}


void 
prefsdlg_delete(GtkWidget* self)
{
	GB_LOG_FUNC
	gtk_widget_hide(self);
	gtk_widget_destroy(self);
	g_free(prefsdlg_xml);
}


void 
prefsdlg_on_ok(GtkButton* button, gpointer user_data)
{
	GB_LOG_FUNC
	g_return_if_fail(prefsdlg_xml != NULL);
	
	PreferencesData *appPrefs =(PreferencesData*)preferences_get();	
	GtkWidget* entry = glade_xml_get_widget(prefsdlg_xml, "tmpDirEntry");
	strncpy(appPrefs->tempDir, gtk_entry_get_text(GTK_ENTRY(entry)), 256);
	
	GtkWidget* checkCleanTmp = 	glade_xml_get_widget(prefsdlg_xml, widget_prefsdlg_cleantempdir);
	appPrefs->cleantempdir = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkCleanTmp));
	
	gboolean oldval = appPrefs->showhiddenfiles;
	GtkWidget* checkShowHidden = glade_xml_get_widget(prefsdlg_xml, widget_prefsdlg_showhidden);
	appPrefs->showhiddenfiles = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkShowHidden));

	/* Trigger the update of the dirtree and filelist if neccessary */
	if (oldval != appPrefs->showhiddenfiles)
	{
		filebrowser_refresh ();
	}

	preferences_save();
}
