/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 /*
 * File: startdlg.c
 * Created by: luke@biddell.co.uk
 * Created on: Wed Apr  7 23:08:35 2004
 */

#include "startdlg.h"
#include "preferences.h"
#include "devices.h"
#include "gnomebaker.h"
#include "gbcommon.h"

void startdlg_on_ok_clicked (GtkButton * button, gpointer user_data);
void startdlg_on_scan (GtkButton * button, gpointer user_data);


GladeXML* startdlg_xml = NULL;


GtkWidget* 
startdlg_new(const BurnType burntype)
{	
	GB_LOG_FUNC
	startdlg_xml = glade_xml_new(glade_file, widget_startdlg, NULL);
	glade_xml_signal_autoconnect(startdlg_xml);
	
	GtkWidget* dlg = glade_xml_get_widget(startdlg_xml, widget_startdlg);
	gtk_window_set_title(GTK_WINDOW(dlg), BurnTypeText[burntype]);
	
	PreferencesData *appPrefs =(PreferencesData *) preferences_get();
	
	GtkWidget *optmenReadDev = glade_xml_get_widget(startdlg_xml, widget_startdlg_reader);
	devices_populate_optionmenu(optmenReadDev);
	
	gint index = devices_get_index_from_scsiid(appPrefs->readDevice);
	gtk_option_menu_set_history(GTK_OPTION_MENU(optmenReadDev), index);
	
	GtkWidget *optmenWriteDev = glade_xml_get_widget(startdlg_xml, widget_startdlg_writer);
	devices_populate_optionmenu(optmenWriteDev);
	
	index = devices_get_index_from_scsiid(appPrefs->writeDevice);
	gtk_option_menu_set_history(GTK_OPTION_MENU(optmenWriteDev), index);
		
	GtkWidget* spinSpeed = glade_xml_get_widget(startdlg_xml, widget_startdlg_speed);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinSpeed), appPrefs->speed);

	GtkWidget* checkDummy = glade_xml_get_widget(startdlg_xml, widget_startdlg_dummy);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkDummy), appPrefs->dummy);

	GtkWidget* checkMultisession = glade_xml_get_widget(startdlg_xml,  widget_startdlg_multisession);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkMultisession), appPrefs->multisession);

	GtkWidget* checkEject = glade_xml_get_widget(startdlg_xml, widget_startdlg_eject);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkEject), appPrefs->eject);
	
	GtkWidget* checkFastErase = glade_xml_get_widget(startdlg_xml, widget_startdlg_fasterase);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkFastErase), appPrefs->fastBlank);
	
	GtkWidget* checkOverburn = glade_xml_get_widget(startdlg_xml, widget_startdlg_overburn);	
	/*gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkOverburn), appPrefs->overburn);*/
	
	GtkWidget* checkBurnFree = glade_xml_get_widget(startdlg_xml, widget_startdlg_burnfree);	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkBurnFree), appPrefs->burnfree);
	
	GtkWidget* checkISOOnly = glade_xml_get_widget(startdlg_xml, widget_startdlg_isoonly);	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkISOOnly), appPrefs->createisoonly);	
	
	GtkWidget *optmenWriteMode = glade_xml_get_widget(startdlg_xml, widget_startdlg_writemode);
	GList* items = GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(optmenWriteMode)))->children;
	index = 0;
	while(items)
	{
		if (GTK_BIN (items->data)->child)
		{
			GtkWidget *child = GTK_BIN (items->data)->child;				
			if (GTK_IS_LABEL (child))
			{
				gchar *text = NULL;			
				gtk_label_get (GTK_LABEL (child), &text);
				if(g_ascii_strcasecmp(text, appPrefs->writeMode) == 0)
					gtk_option_menu_set_history(GTK_OPTION_MENU(optmenWriteMode), index);	
			}
		}
		items = items->next;
		++index;
	}
		
	switch(burntype)
	{		
		case blank_cdrw:		
			gtk_widget_hide(checkDummy);
			gtk_widget_hide(checkMultisession);
			gtk_widget_hide(checkOverburn);
			gtk_widget_hide(optmenReadDev);
			gtk_widget_hide(glade_xml_get_widget(startdlg_xml, widget_startdlg_readlabel));
			gtk_widget_hide(glade_xml_get_widget(startdlg_xml, widget_startdlg_modelabel));
			gtk_widget_hide(optmenWriteMode);
			gtk_widget_hide(checkBurnFree);
			gtk_widget_hide(checkISOOnly);
			gtk_widget_show(checkFastErase);
			break;				
		case create_audio_cd:
		case create_mixed_cd:
		case burn_cd_image:			
			gtk_widget_hide(checkISOOnly);
		case create_video_cd:			
		case create_data_cd:		
			gtk_widget_hide(optmenReadDev);
			gtk_widget_hide(glade_xml_get_widget(startdlg_xml, widget_startdlg_readlabel));
			break;
		case copy_audio_cd:
			gtk_widget_hide(checkISOOnly);
		case copy_data_cd:		
		default:
			break;
	};
	
	return dlg;
}


void 
startdlg_delete(GtkWidget* self)
{	
	GB_LOG_FUNC
	gtk_widget_hide(self);
	gtk_widget_destroy(self);
	g_free(startdlg_xml);
}


void 
startdlg_on_ok_clicked(GtkButton * button, gpointer user_data)
{
	GB_LOG_FUNC
	PreferencesData *appPrefs =(PreferencesData *) preferences_get();
	
	GtkWidget* optmenReadDev = glade_xml_get_widget(startdlg_xml, widget_startdlg_reader);
	const gchar *scsiid = devices_get_scsiid_from_index(
		gtk_option_menu_get_history(GTK_OPTION_MENU(optmenReadDev)));
	if(scsiid)
		strncpy(appPrefs->readDevice, scsiid, 64);

	GtkWidget* optmenWriteDev = glade_xml_get_widget(startdlg_xml, widget_startdlg_writer);
	scsiid = devices_get_scsiid_from_index(
		gtk_option_menu_get_history(GTK_OPTION_MENU(optmenWriteDev)));
	if(scsiid)
		strncpy(appPrefs->writeDevice, scsiid, 64);

	appPrefs->speed = gtk_spin_button_get_value(
		GTK_SPIN_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_speed)));

	appPrefs->dummy = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_dummy)));
	
	appPrefs->eject = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_eject)));
	
	appPrefs->multisession = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_multisession)));
	
	appPrefs->fastBlank = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_fasterase)));
	
	appPrefs->burnfree = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_burnfree)));
	
	appPrefs->createisoonly = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(glade_xml_get_widget(startdlg_xml, widget_startdlg_isoonly)));
		
	GtkWidget* optmenWriteMode = glade_xml_get_widget(startdlg_xml, widget_startdlg_writemode);
	GtkWidget* mode = GTK_BIN(optmenWriteMode)->child;
	if(mode != NULL && GTK_IS_LABEL(mode))
	{
		gchar *text = NULL;
		gtk_label_get(GTK_LABEL(mode), &text);
		strncpy(appPrefs->writeMode, text, 64);	
		/*g_free(text);*/
	}
						
	preferences_save();	
}


void 
startdlg_on_scan(GtkButton * button, gpointer user_data)
{
	GB_LOG_FUNC
	gbcommon_start_busy_cursor1(startdlg_xml, widget_startdlg);
	
	if(devices_probescsibus())
	{
		PreferencesData *appPrefs =(PreferencesData *) preferences_get();
		
		GtkWidget *optmenReadDev = glade_xml_get_widget(startdlg_xml, widget_startdlg_reader);
		devices_populate_optionmenu(optmenReadDev);
		
		gint index = devices_get_index_from_scsiid(appPrefs->readDevice);
		gtk_option_menu_set_history(GTK_OPTION_MENU(optmenReadDev), index);
		
		GtkWidget *optmenWriteDev = glade_xml_get_widget(startdlg_xml, widget_startdlg_writer);
		devices_populate_optionmenu(optmenWriteDev);
		
		index = devices_get_index_from_scsiid(appPrefs->writeDevice);
		gtk_option_menu_set_history(GTK_OPTION_MENU(optmenWriteDev), index);	
	}		
	
	gbcommon_end_busy_cursor1(startdlg_xml, widget_startdlg);
}
